import iris
import matplotlib.pyplot as plt
import plot_functions as pf
import cartopy.crs as ccrs
import numpy as np
import datetime as dt 
import iris.coord_categorisation as icc
import field_functions as ff
import numpy.ma as ma
import regions
import cube_magic as cm
import pylab
from matplotlib.font_manager import FontProperties
import copy
import regrid
from string import ascii_lowercase
from arrays import wrap

mode = 'read'
sf = ff.sample_field('n96x')
lat_coord = sf.coord('latitude')
lon_coord = sf.coord('longitude')
region_name = 'Arctic_Ocean.0'
gridname_itd = 'hadgom'
gridname_surface = 'n96x'
gridname_sf_ref = 'n96x'
models_list = [['UKESM1.0-N96ORCA1','HadGEM2_ES'],
               ['HadGEM3_GC3.1_new','HadGEM2_ES'],
               ['UKESM1.0-N96ORCA1','HadGEM3_GC3.1_new']]

model_titles = {'HadGEM2_ES':'HadGEM2-ES',
        'HadGEM3_GC3.1_new':'HadGEM3-GC3.1-LL',
        'UKESM1.0-N96ORCA1':'UKESM1.0-LL'}

savedir = '/data/users/hadax/PhD/Papers/Dec2019_HadGEM3_UKESM/v3_29_03_2021/'+\
    'plot_isf_contributions_less_info/'
sample_field_itd = ff.sample_field(gridname_itd)
sample_field_surface = ff.sample_field(gridname_surface)
sample_field_sf_ref = ff.sample_field(gridname_sf_ref)

nx_surface = sample_field_surface.shape[1]
ny_surface = sample_field_surface.shape[0]

sf_gridnames = {'HadGEM2_ES': 'n96','HadGEM3_GC3.1_new': 'n96x',
    'UKESM1.0-N96ORCA1': 'n96x'}

snapshots = [45,60,75,90,105,120]

colours = {'SW_down':'#ff8000','LW_down':'#804000',
    'Total_CRE':'#00ffa0',
    'Ice_area':'#0000ff',\
    'Snow_presence':'#ff8f8f','Melt_occurrence':'#ff0000',\
           'Mpond_scheme':'#808080','Snow_param':'#800080',
           'Sensible_heat_flux':'#800000','Latent_heat_flux':'#800080',\
           'Ice_thickness':'#ff00ff','Snow_thickness':'#000080',
           'Ice export':'#00af00'}

linestyles = {'SW_down':(1,0),
    'LW_down':(1,0),
    'Snow_presence':(),
    'Snow_param':(2,2),
    'Melt_occurrence':(2,2,6,2),
    'Mpond_scheme':(2,2,6,2),
              'Ice_area':(6,2),
              'Ice_thickness':(6,2),
              'Snow_thickness':(6,2),
              'Sensible_heat_flux':(2,2,6,2),
              'Latent_heat_flux':(2,2,6,2),
              'Ice export':(2,0)}

markers = {'SW_down':'*',
    'LW_down':'s',
    'Snow_presence':'o',
    'Snow_param':'^',
    'Melt_occurrence':'*',
    'Mpond_scheme':'s',
              'Ice_area':'o',
              'Ice_thickness':'^',
              'Snow_thickness':'*',
              'Sensible_heat_flux':'s',
              'Latent_heat_flux':'o',
              'Ice export':'^'}

titles = {'SW_down':'Downwelling SW',
          'LW_down':'Downwelling LW',
          'Total_CRE':'Total cloud radiative effect',
          'Ice_area':'Sea ice concentration',
          'Ice_thickness':'Sea ice thickness distribution',
          'Snow_thickness':'Snow thickness insulation effect',
          'Snow_presence':'Snow thickness albedo effect',
          'Snow_param':'Snow area parameterisation',
          'Melt_occurrence':'Surface melt onset',
          'Mpond_scheme':'Melt pond parameterisation',
          'Sensible_heat_flux':'Sensible heat flux',
          'Latent_heat_flux':'Latent heat flux'}

component_names = ['LW_down_CS','SW_down_CS','LW_CRE','SW_CRE','Sensible_heat_flux','Latent_heat_flux','Ice_area','Snow_presence',\
          'Melt_occurrence',\
                   'Mpond_scheme','Snow_param']
component_names_plot = component_names + ['Ice_thickness','Snow_thickness','SW_down','LW_down']
component_names_plot.remove('LW_CRE')
component_names_plot.remove('SW_CRE')
component_names_plot.remove('SW_down_CS')
component_names_plot.remove('LW_down_CS')
component_names_plot.remove('Sensible_heat_flux')
component_names_plot.remove('Latent_heat_flux')

startyear = 1980
endyear = 1999
tcode = str(startyear) + '.' + str(endyear)

outfile = '/home/h01/hadax/graphics/PhD/Papers/Dec2019_HadGEM3_UKESM/v4_reviewer_response_10_02_2022/monthly_isf_contributions_extend_less_info_with_proxies.'+region_name+'.png'

ref_date = dt.date(1978,9,1)
ref_units = ref_date.strftime('days since %Y-%m-%d')

n_years = endyear - startyear + 1
years = np.arange(startyear,endyear+1)
months = np.arange(1,13)
months = np.tile(months,n_years)
years = np.repeat(years,12)
time_points = np.array([dt.date(year,month,1).toordinal() - \
    ref_date.toordinal() for (year,month) in zip(years,months)])
time_coord = iris.coords.DimCoord(time_points,'time',units=ref_units)

all_data_1d_monthly = {}
all_total_monthly = {}
all_sf_diff_1D = {}
for models in models_list:
    model_label = models[0] + '_vs_' + models[1]
    if mode == 'read':
        all_data_1d_monthly[model_label] = {}
        all_sf_diff_1D[model_label] = {}
        for varname in component_names_plot:
            savefile = savedir + model_label + '.' + varname + '.' + tcode + '.' + \
            region_name + '.npz'
            with np.load(savefile) as filehz:
                all_data_1d_monthly[model_label][varname] = \
                    ma.masked_array(**filehz)

        savefile = savedir + model_label + '.' + 'total_isf_diff' + '.' + tcode + '.' + \
            region_name + '.npz'
        with np.load(savefile) as filehz:
            all_total_monthly[model_label] = ma.masked_array(**filehz)

        savefile = savedir + model_label + '.' + 'total_surface_flux_diff' + '.' + tcode + '.' + \
            region_name + '.npz'
        with np.load(savefile) as filehz:
            all_sf_diff_1D[model_label]['Total_surface_flux'] = ma.masked_array(**filehz)

    elif mode == 'calc':

        basedir = '/data/users/hadax/PhD/daily_albedo_extend/'+models[0]+'_vs_'+models[1]+'/'

        print(' ')
        print(' ')
        print('=======================================')
        print('Reading in data for model combination '+model_label)
        print(' ')

        # Read surface flux components data
        data = {}
        data_1d = {}
        print('Reading surface flux components data')
        for varname in component_names:
            cubelist = iris.cube.CubeList([])
            data_array = ma.masked_array(np.zeros((360,ny_surface,nx_surface)),\
                                         mask=np.zeros((360,ny_surface,nx_surface),dtype='bool'))
            count = np.zeros((360,ny_surface,nx_surface))
            for year in range(startyear,endyear+1):
                filename = 'isf.' + varname + '.' + str(year) + '.nc'
                ffile = basedir + filename
                ccube = iris.load_cube(ffile)

                data_array.data[:] = data_array.data[:] + ccube.data.data[:]
                count = count + 1. - ccube.data.mask.astype('int')

            data_array = data_array / count
            index = np.where(np.abs(data_array) > 1.e10)
            data_array.mask[index] = True

            data[varname] = iris.cube.Cube(data_array)
            data[varname].add_dim_coord(lon_coord,2)
            data[varname].add_dim_coord(lat_coord,1)
            regions.reduce_to_region(data[varname],region_name,'n96x')

            cm.llbounds(data[varname])
            aw = iris.analysis.cartography.area_weights(data[varname])
            data_1d[varname] = data[varname].collapsed(['latitude','longitude'],\
                 iris.analysis.MEAN,weights=aw)

        print('Reading in total surface flux data')
        # Read in total surface flux data for comparison
        basedir_surface_flux = '/data/users/hadax/PhD/multiannual_fields/'
        sf_1D = {}
        sf_3D = {}
        vars_read_cf = ['SW_down_CS','SW_CRE','LW_down_CS','LW_CRE','SW_up','LW_up',
                        'Sensible_heat_flux','Latent_heat_flux','Total_surface_flux']
        for model in models:
            fulldir = basedir_surface_flux + model + '/Hist1/SFC/monthly/'

            sf_1D[model] = {}
            for var in vars_read_cf:
                filename = var + '.' + tcode + '.MEAN.nc'
                sf_file = fulldir + filename
                sf_cube = iris.load_cube(sf_file)
                regions.reduce_to_region(sf_cube,region_name,sf_gridnames[model])

                if cm.latlon_is_2D(sf_cube):
                    import mesh
                    aw2 = mesh.area_weights(sf_gridnames[model])
                    aw = np.tile(aw2,(12,1,1))
                else:
                    cm.llbounds(sf_cube)
                    aw = iris.analysis.cartography.area_weights(sf_cube)

                ts_sf_cube = sf_cube.collapsed(['latitude','longitude'],iris.analysis.MEAN,
                    weights=aw)

                sf_1D[model][var] = ts_sf_cube


        sf_diff_1D = {}
        for var in vars_read_cf:

            sf_diff_1D[var] = sf_1D[models[0]][var] - sf_1D[models[1]][var]


        total1 = np.zeros(360)
        for varname in component_names:
            total1 = total1 + data_1d[varname].data


        data_1d_monthly = {}
        total_monthly = np.zeros(12)
        for varname in [*data_1d.keys()]:
            array = np.reshape(data_1d[varname].data,(12,30))
            array_m = np.mean(array,axis=1)
            data_1d_monthly[varname] = array_m
            total_monthly = total_monthly + array_m

        data_1d_monthly['SW_down'] = data_1d_monthly['SW_down_CS'] + \
            data_1d_monthly['SW_CRE']

        data_1d_monthly['LW_down'] = data_1d_monthly['LW_down_CS'] + \
            data_1d_monthly['LW_CRE']

        print('Reading in ITD data')
        # Read in ITD data
        basedir = '/data/users/hadax/PhD/ISF_intermodel/'
        datadir = basedir + models[0] + '_vs_' + models[1] + '.' + gridname_itd + '/'

        varnames_read = ['lw_down','sw_net','shf_ice'] + \
            ['aicen.cat.' + str(cat) for cat in range(1,6)] + \
            ['hicen.cat.' + str(cat) for cat in range(1,6)] + \
            ['hsnon.cat.' + str(cat) for cat in range(1,6)]

        data1 = {}
        for varname in varnames_read:
            all_cubelist = iris.cube.CubeList([])
            for year in range(startyear,endyear+1):
                ffile = datadir + varname + '.' + str(year) + '.nc'
                ccube = iris.load_cube(ffile)
                all_cubelist.append(ccube)

            for c in all_cubelist:
                c.attributes = {}
            all_cubes = all_cubelist.concatenate()
            if len(all_cubes) > 1:
                print('Unable to concatenate cubes')
                raise ValueError

            all_cube = all_cubes[0]
            icc.add_month(all_cube,'time')
            sc_cube = all_cube.aggregated_by('month',iris.analysis.MEAN)
            sc_cube_rg = regrid.regrid2(sc_cube,gridname_sf_ref)
            sc_cube_rg.add_aux_coord(sc_cube.coord('month'),0)
            regions.reduce_to_region(sc_cube,region_name,gridname_itd)

            cm.llbounds(sc_cube)
            aw = iris.analysis.cartography.area_weights(sc_cube)

            sc_cube1 = sc_cube.collapsed(['latitude','longitude'],iris.analysis.MEAN,\
                weights=aw)
            data1[varname] = cm.sc_translate_1D(sc_cube1).data

        print('Computing totals')
        for varname in ['aicen','hicen','hsnon']:
            ny_ref = sample_field_sf_ref.shape[0]
            nx_ref = sample_field_sf_ref.shape[1]

            count1 = ma.zeros(12)
            for cat in range(1,6):
                count1 = count1 + data1[varname+'.cat.'+str(cat)]

            data1[varname+'.TOTAL'] = count1

        for cat in range(1,6):
            data1['ice.cat.'+str(cat)] = data1['aicen.cat.'+str(cat)] + \
                                         data1['hicen.cat.'+str(cat)]

            data1['ice_and_snow.cat.'+str(cat)] = data1['aicen.cat.'+str(cat)] + \
                                         data1['hicen.cat.'+str(cat)] + \
                                         data1['hsnon.cat.'+str(cat)]

        data_1d_monthly['Ice_thickness'] =  data1['aicen.TOTAL'] + data1['hicen.TOTAL']
        data_1d_monthly['Snow_thickness'] = data1['hsnon.TOTAL']
        data_1d_monthly['Total_CRE'] = data_1d_monthly['SW_CRE'] + data_1d_monthly['LW_CRE']

        total_monthly = total_monthly + data_1d_monthly['Ice_thickness']
        total_monthly = total_monthly + data_1d_monthly['Snow_thickness']

        all_data_1d_monthly[model_label] = data_1d_monthly
        all_total_monthly[model_label] = total_monthly
        all_sf_diff_1D[model_label] = sf_diff_1D

        for varname in [*all_data_1d_monthly[model_label].keys()]:
            savefile = savedir + model_label + '.' + varname + '.' + tcode + '.' + \
                region_name + '.npz'
            np.savez_compressed(savefile,data=all_data_1d_monthly[model_label][varname].data,mask=all_data_1d_monthly[model_label][varname].mask)

        savefile_total = savedir + model_label + '.total_isf_diff.' + tcode + '.' + \
                region_name + '.npz'
        np.savez_compressed(savefile_total,data=all_total_monthly[model_label].data,mask=all_total_monthly[model_label].mask)
        savefile_actual_total = savedir + model_label + '.total_surface_flux_diff.' + tcode + '.' + \
                region_name + '.npz'
        np.savez_compressed(savefile_actual_total,data=all_sf_diff_1D[model_label]['Total_surface_flux'].data.data,mask=all_sf_diff_1D[model_label]['Total_surface_flux'].data.mask)


# Attempt to plot proxy quantities
proxy_netrad_diff = {}
proxy_ice_growthmelt_diff = {}
rho_ice = 917.
rho_snow = 330.
qfus = 3.35e5
seconds_in_month = 8.64e4 * 30.
ice_gridnames = {'HadGEM2_ES':'hadgom','HadGEM3_GC3.1_new':'eorca1','UKESM1.0-N96ORCA1':'eorca1'}
for models in models_list:
    proxy_netrad = {}
    proxy_ice_growthmelt = {}
    for model in models:
        proxy_datadir = '/data/users/hadax/PhD/ISF/hypermeans/v10_01_04_2021/' + \
            model + '/monthly/' + ice_gridnames[model] + '/'
        sw_filename = 'SW_net' + '.' + 'Hist1' + '.' + tcode + '.nc'
        lw_filename = 'LW_net' + '.' + 'Hist1' + '.' + tcode + '.nc'
        hice_filename = 'Ice_volume' + '.' + 'Hist1' + '.' + tcode + '.nc'
        hsno_filename = 'Snow_volume' + '.' + 'Hist1' + '.' + tcode + '.nc'
        sw_cube = iris.load_cube(proxy_datadir+sw_filename)
        lw_cube = iris.load_cube(proxy_datadir+lw_filename)
        hice_cube = iris.load_cube(proxy_datadir+hice_filename)
        hsno_cube = iris.load_cube(proxy_datadir+hsno_filename)

        netrad_cube = sw_cube + lw_cube
        ice_hc_cube = (hice_cube * rho_ice + hsno_cube * rho_snow) * qfus / seconds_in_month

        regions.reduce_to_region(netrad_cube,region_name,ice_gridnames[model])
        regions.reduce_to_region(ice_hc_cube,region_name,ice_gridnames[model])

        if len(netrad_cube.coord('latitude').points.shape)==2:
            import mesh
            aw2 = mesh.area_weights_2D(ice_gridnames[model])
            aw = np.tile(aw2,(netrad_cube.shape[0],1,1))
        else:
            import cube_magic as cm
            cm.llbounds(netrad_cube)
            aw = iris.analysis.cartography.area_weights(netrad_cube)

        ts_netrad = netrad_cube.collapsed(['latitude','longitude'],
            iris.analysis.MEAN,weights=aw)
        ts_ice_hc = ice_hc_cube.collapsed(['latitude','longitude'],
            iris.analysis.MEAN,weights=aw)

        try:
            icc.add_month(ts_netrad,'time')
        except ValueError:
            pass
        sc_netrad = ts_netrad.aggregated_by('month',iris.analysis.MEAN)
        try:
            icc.add_month(ts_ice_hc,'time')
        except ValueError:
            pass

        sc_ice_hc = ts_ice_hc.aggregated_by('month',iris.analysis.MEAN)
        
        sc_ice_hu = copy.deepcopy(sc_ice_hc)
        sc_ice_hu.data = (np.roll(sc_ice_hc.data,1) - np.roll(sc_ice_hc.data,-1)) / 2.

        proxy_netrad[model] = sc_netrad
        proxy_ice_growthmelt[model] = sc_ice_hu

    model_label = models[0] + '_vs_' + models[1]

    proxy_netrad_diff[model_label] = proxy_netrad[models[0]].data - proxy_netrad[models[1]].data
    proxy_ice_growthmelt_diff[model_label] = proxy_ice_growthmelt[models[0]].data - proxy_ice_growthmelt[models[1]].data



# Read ice divergence terms
icediv_scale_factors = {'UKESM1.0-N96ORCA1':3.35e5*917./(8.64e4*100.),
    'HadGEM3_GC3.1_new':3.35e5*917./(8.64e4*100.),
    'HadGEM2_ES':3.35e5*917.}
icediv_dir = '/data/users/hadax/PhD/timeseries/'
icediv_data = {}
for model in [*model_titles.keys()]:
    subdir = icediv_dir + model + '/Hist1/ICE_TEND/monthly/'
    icediv_filename = 'ice_volume_tendency_dynamics.'+tcode+'.'+region_name+'.MEAN.nc'
    icediv_cube = iris.load_cube(subdir+icediv_filename)
    icediv_cube = icediv_cube * icediv_scale_factors[model] * -1.
    icc.add_month(icediv_cube,'time')
    sc_cube = icediv_cube.aggregated_by('month',iris.analysis.MEAN)
    icediv_data[model] = sc_cube

icediv_diff = {}
for models in models_list:
    model_label = models[0] + '_vs_' + models[1]
    icediv_cfg = copy.deepcopy(icediv_data[models[0]])
    icediv_cfg.data = icediv_data[models[1]].data
    icediv_diff[model_label] = icediv_data[models[0]] - icediv_cfg

all_total_monthly[model_label] = all_total_monthly[model_label] + icediv_diff[model_label]


ylim = [-42,17]
xl = .065
yb = .21
ys = .27
yw = .76
xw = .85

xplot_m = np.arange(-.5,13,1.)
xoffset = 0.

font_large = FontProperties(size=15)

figure = plt.figure(figsize=(14,12))

models_fullplot = models_list[0]
imodels = 0
model_label = models_fullplot[0] + '_vs_' + models_fullplot[1]
model_title = model_titles[models_fullplot[0]] + ' vs ' + model_titles[models_fullplot[1]]
axis = figure.add_axes([xl,yb-ys*imodels,xw,yw])
for (ivar,varname) in enumerate(component_names_plot):
    line, = axis.plot(xplot_m,wrap(all_data_1d_monthly[model_label][varname].data),label=titles[varname],\
              color = colours[varname],marker=markers[varname],markersize=10)
    line.set_dashes(linestyles[varname])

line, = axis.plot(xplot_m,wrap(icediv_diff[model_label].data),label='Ice export',
    color = colours['Ice export'],marker=markers['Ice export'],markersize=10)
line.set_dashes(linestyles['Ice export'])

axis.plot(xplot_m,wrap(all_total_monthly[model_label]),label='TOTAL',color='k',
          linewidth=3,marker='*',markersize=10)
#axis.bar(xplot_m+xoffset*(9+1.7),all_sf_diff_1D[model_label]['Total_surface_flux'].data,width=0.07,label='Actual surface flux difference',color='#808080')

axis.plot(xplot_m,wrap(proxy_netrad_diff[model_label]),label='Net radiation difference',marker='*',markersize=10,linestyle='--',color='#808080',linewidth=3)
line, = axis.plot(xplot_m,wrap(proxy_ice_growthmelt_diff[model_label]),label='Ice melt/growth difference',marker='*',markersize=10,color='#808080',linewidth=3)

line.set_dashes((6,1,1,1))
axis.plot([0,12],[0,0],color='#c0c0c0',linestyle='--')

axis.set_ylabel('Flux difference ($Wm^{-2}$)',fontproperties=font_large)
axis.set_xticks(np.arange(0,7,1))
for month in range(1,13):
    month_name = dt.date(1980,month,1).strftime('%b')
    axis.text(month-.5,ylim[0] - .03*(ylim[1]-ylim[0]),\
              str(month),ha='center',fontproperties=font_large)
    plt.plot([month]*2,ylim,color='#c0c0c0')

axis.set_xlabel('Month',labelpad=20,fontproperties=font_large)
for label in axis.get_xticklabels()+axis.get_yticklabels():
    label.set_fontproperties(font_large)

axis.set_ylim(ylim)
axis.set_xlim(0,12)

axis.legend(prop=font_large,bbox_to_anchor=[-.07,-.27,.5,.5],ncol=3,
            handlelength=5)

axis.set_title(model_title,fontproperties=font_large)
ax2 = axis.twinx()
ax2.set_ylabel('Ice growth/melt difference (cm)',fontproperties=font_large)
growthmelt_scale = 100. * 30. * 8.64e4 / (917.  * 3.35e5) * -1.
ax2.set_ylim(ylim[1] * growthmelt_scale,ylim[0] * growthmelt_scale)
ax2.invert_yaxis()

for label in ax2.get_yticklabels():
    label.set_fontproperties(font_large)

for label in axis.get_xticklabels():
    label.set_visible(False)

plot_extra=False
if plot_extra:
    xl_j = xl
    yb_j = .05
    xw_j = .2
    xs_j = xw - xw_j
    yw_j = .21

    xplot_june = 5.12
    xoffset_june = .08


    for imodels in range(1,3):
        models_juneplot = models_list[imodels]
        model_label = models_juneplot[0] + '_vs_' + models_juneplot[1]
        model_title = '('+ascii_lowercase[imodels]+') '+ model_titles[models_juneplot[0]] + ' vs ' + model_titles[models_juneplot[1]]
        axis = figure.add_axes([xl_j+xs_j*(imodels-1),yb_j,xw_j,yw_j])
        for (ivar,varname) in enumerate(component_names_plot):
            axis.bar(xplot_june+xoffset_june*ivar,all_data_1d_monthly[model_label][varname].data[5],width=0.04,label=titles[varname],\
                 color = colours[varname],edgecolor='k')

        axis.bar(xplot_june+xoffset_june*(9+.5),all_total_monthly[model_label][5],width=0.07,label='TOTAL',color='k')
        #axis.bar(xplot_june+xoffset*(9+1.7),all_sf_diff_1D[model_label]['Total_surface_flux'].data[5],width=0.07,label='Actual surface flux difference',color='#808080')
        axis.plot([0,12],[0,0],color='#c0c0c0',linestyle='--')
        axis.set_title(model_title,fontproperties=font_large,pad=20)
        axis.set_ylim(ylim)
        axis.set_ylabel('Flux difference ($Wm^{-2}$)',fontproperties=font_large)
        axis.set_xticks(np.arange(0,7,1))
        for month in range(6,7):
            month_name = dt.date(1980,month,1).strftime('%b')
            axis.text(month-.5,ylim[0] - .04*(ylim_j[1]-ylim_j[0]),\
                      str(month),ha='center',fontproperties=font_large)
            plt.plot([month]*2,ylim,color='#c0c0c0')

        axis.set_xlabel('Month',labelpad=20)
        axis.set_xlim(5,6)
        for label in axis.get_xticklabels()+axis.get_yticklabels():
            label.set_fontproperties(font_large)

        axis.set_ylim(ylim_j)
        axis.set_xlim(5,6)
        for label in axis.get_xticklabels():
            label.set_visible(False)

pylab.savefig(outfile)

